﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;
using Infragistics.Win.UltraWinEditors;
using Infragistics.Win;

namespace UOMEditorSample
{
	public class UOMEditor : UltraNumericEditor
	{
		public UOMEditor()
			: base()
		{
		}

		protected override void OnEndInit()
		{
			base.OnEndInit();
			if (!DesignMode)
			{
				this.DataFilter = new UOMDataFilter(this);
			}
		}

		private MeasureSystemType measureSystem = MeasureSystemType.English;

		[Description("Gets or sets the display unit type")]
		[Category("Behavior")]
		[DefaultValue(MeasureSystemType.English)]
		public MeasureSystemType MeasureSystem
		{
			get
			{
				return measureSystem;
			}
			set
			{
				measureSystem = value;
				this.mainElement.DirtyChildElements();
			}
		}

		private string UnitSymbol
		{
			get
			{
				if (MeasureSystem == MeasureSystemType.English)
				{
					return "\"";
				}
				else if (MeasureSystem == MeasureSystemType.Metric)
				{
					return "mm";
				}
				return null;
			}
		}

		public override string MaskInput
		{
			get
			{
				StringBuilder sb = new StringBuilder();
				sb.Append(base.MaskInput);

				if (sb.Length > 0)
				{
					// Only append the symbol at runtime, not on the designer
					if (!DesignMode)
					{
						sb.Append(" ").Append(UnitSymbol);
					}
					return sb.ToString();
				}
				return null;
			}
			set
			{
				base.MaskInput = value;
			}
		}

		private double ConvertToDisplay(double sourceValue)
		{
			if (this.MeasureSystem == MeasureSystemType.Metric)
			{
				return sourceValue * 25.4;
			}
			return sourceValue;
		}

		private double ConvertFromDisplay(double sourceValue)
		{
			if (this.MeasureSystem == MeasureSystemType.Metric)
			{
				return sourceValue / 25.4;
			}
			return sourceValue;
		}

		public enum MeasureSystemType
		{
			English,
			Metric
		}

		private class UOMDataFilter : Infragistics.Win.IEditorDataFilter
		{
			private UOMEditor parent;

			public UOMDataFilter(UOMEditor parent)
			{
				if (parent == null)
				{
					throw new ArgumentNullException("parent");
				}
				this.parent = parent;
			}

			#region IEditorDataFilter Members

			public object Convert(EditorDataFilterConvertArgs conversionArgs)
			{
				switch (conversionArgs.Direction)
				{
					case ConversionDirection.OwnerToEditor:
						if (conversionArgs.Value != DBNull.Value
							&& conversionArgs.Value != null)
						{
							double doubleValue = System.Convert.ToDouble(conversionArgs.Value);
							conversionArgs.Handled = true;
							return parent.ConvertToDisplay(doubleValue);
						}
						break;
					case ConversionDirection.EditorToOwner:
						if (conversionArgs.Value != DBNull.Value && conversionArgs.Value != null)
						{
							double doubleValue = System.Convert.ToDouble(conversionArgs.Value);
							conversionArgs.Handled = true;
							return parent.ConvertFromDisplay(doubleValue);
						}
						break;
				}
				return conversionArgs.Value;
			}
			#endregion
		}
	}
}
